import logging

import flask_login
from flask import Blueprint, render_template, request
import flask
import werkzeug.exceptions as wz_exceptions

import pillarsdk
import pillar.api.utils
from pillar.web.system_util import pillar_api

from attract.routes import attract_project_view
from attract.node_types.shot import node_type_shot, task_types
from attract import current_attract, ROLES_REQUIRED_TO_VIEW_ITEMS
from pillar.web.utils import get_file

from . import routes_common

perproject_blueprint = Blueprint('attract.shots.perproject', __name__,
                                 url_prefix='/<project_url>/shots')
log = logging.getLogger(__name__)


@perproject_blueprint.route('/', endpoint='index')
@perproject_blueprint.route('/with-task/<task_id>', endpoint='with_task')
@attract_project_view(extension_props=True)
def for_project(project, attract_props, task_id=None, shot_id=None):
    shots, tasks_for_shots, task_types_for_template = routes_common.for_project(
        node_type_shot['name'],
        task_types,
        project, attract_props, task_id, shot_id)

    # Some aggregated stats
    stats = {
        'nr_of_shots': sum(shot.properties.used_in_edit or 0 for shot in shots),
        'total_frame_count': sum(shot.properties.duration_in_edit_in_frames or 0
                                 for shot in shots
                                 if shot.properties.used_in_edit),
    }

    return render_template('attract/shots/for_project.html',
                           shots=shots,
                           tasks_for_shots=tasks_for_shots,
                           task_types=task_types_for_template,
                           open_task_id=task_id,
                           open_shot_id=shot_id,
                           project=project,
                           attract_props=attract_props,
                           stats=stats)


@perproject_blueprint.route('/<shot_id>')
@attract_project_view(extension_props=True)
def view_shot(project, attract_props, shot_id):
    if not request.is_xhr:
        return for_project(project, attract_props, shot_id=shot_id)

    shot, node_type = routes_common.view_node(project, shot_id, node_type_shot['name'])

    return render_template('attract/shots/view_shot_embed.html',
                           shot=shot,
                           project=project,
                           shot_node_type=node_type,
                           attract_props=attract_props,
                           can_edit='PUT' in shot.allowed_methods)


@perproject_blueprint.route('/<shot_id>', methods=['POST'])
@attract_project_view()
def save(project, shot_id):
    log.info('Saving shot %s', shot_id)
    log.debug('Form data: %s', request.form)

    shot_dict = request.form.to_dict()
    current_attract.shot_manager.edit_shot(shot_id, **shot_dict)

    # Return the patched node in all its glory.
    api = pillar_api()
    shot = pillarsdk.Node.find(shot_id, api=api)
    return pillar.api.utils.jsonify(shot.to_dict())


# TODO: remove GET method once Pablo has made a proper button to call this URL with a POST.
@perproject_blueprint.route('/create', methods=['POST', 'GET'])
@attract_project_view()
def create_shot(project):
    shot = current_attract.shot_manager.create_shot(project)

    resp = flask.make_response()
    resp.headers['Location'] = flask.url_for('.view_shot',
                                             project_url=project['url'],
                                             shot_id=shot['_id'])
    resp.status_code = 201
    return flask.make_response(flask.jsonify({'shot_id': shot['_id']}), 201)


@perproject_blueprint.route('/<shot_id>/activities')
@attract_project_view()
def activities(project, shot_id):
    if not request.is_xhr:
        return flask.redirect(flask.url_for('.view_shot',
                                            project_url=project.url,
                                            shot_id=shot_id))

    acts = current_attract.activities_for_node(shot_id)
    return flask.render_template('attract/shots/view_activities_embed.html',
                                 activities=acts)
